@@ -375,7 +375,8 @@ class Job:
             raise AttributeError("The escrow has been already deployed.")
 
         # Use factory to deploy a new escrow contract.
-        self._create_escrow()
+        trusted_handlers = [addr for addr, priv_key in self.multi_credentials]
+        self._create_escrow(trusted_handlers)
         job_addr = self._last_escrow_addr()
         LOG.info("Job's escrow contract deployed to:{}".format(job_addr))
         self.job_contract = get_escrow(job_addr)
@@ -404,6 +405,12 @@ class Job:
 
         >>> job.launch(rep_oracle_pub_key)
         True
+        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
+        >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.gas_payer_priv = "657b6497a355a3982928d5515d48a84870f057c4d16923eb1d104c0afada9aa8"
+        >>> job.multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
         >>> job.setup()
         True
 
@@ -425,29 +432,86 @@ class Job:
             self.serialized_manifest["recording_oracle_addr"])
         hmt_amount = int(self.amount * 10**18)
 
+        hmt_transferred = False
+        contract_is_setup = False
+
+        txn_info = {
+            "gas_payer": self.gas_payer,
+            "gas_payer_priv": self.gas_payer_priv,
+            "gas": gas
+        }
+
         # Fund the escrow contract with HMT.
         hmtoken_contract = get_hmtoken()
-        txn_func = hmtoken_contract.functions.transfer
-        func_args = [self.job_contract.address, hmt_amount]
-        txn_info = {
-            "gas_payer": self.gas_payer,
-            "gas_payer_priv": self.gas_payer_priv,
-            "gas": gas
-        }
-        handle_transaction(txn_func, *func_args, **txn_info)
+        try:
+            txn_func = hmtoken_contract.functions.transfer
+            func_args = [self.job_contract.address, hmt_amount]
+            handle_transaction(txn_func, *func_args, **txn_info)
+            hmt_transferred = True
+        except Exception as e:
+            LOG.error(
+                f"Transferring HMT failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            )
+
+        if not hmt_transferred:
+            for gas_payer, gas_payer_priv in self.multi_credentials:
+                txn_info = {
+                    "gas_payer": gas_payer,
+                    "gas_payer_priv": gas_payer_priv,
+                    "gas": gas
+                }
+
+                try:
+                    txn_func = hmtoken_contract.functions.transfer
+                    func_args = [self.job_contract.address, hmt_amount]
+                    handle_transaction(txn_func, *func_args, **txn_info)
+                    break
+                except Exception as e:
+                    LOG.error(
+                        f"Transferring HMT failed with: {self.gas_payer} and {self.gas_payer_priv} due to {e}. Raffling new ones..."
+                    )
 
-        # Setup the escrow contract with manifest and IPFS data.
-        txn_func = self.job_contract.functions.setup
-        func_args = [
-            reputation_oracle, recording_oracle, reputation_oracle_stake,
-            recording_oracle_stake, self.manifest_url, self.manifest_hash
-        ]
         txn_info = {
             "gas_payer": self.gas_payer,
             "gas_payer_priv": self.gas_payer_priv,
             "gas": gas
         }
-        handle_transaction(txn_func, *func_args, **txn_info)
+
+        try:
+            # Setup the escrow contract with manifest and IPFS data.
+            txn_func = self.job_contract.functions.setup
+            func_args = [
+                reputation_oracle, recording_oracle, reputation_oracle_stake,
+                recording_oracle_stake, self.manifest_url, self.manifest_hash
+            ]
+            handle_transaction(txn_func, *func_args, **txn_info)
+            contract_is_setup = True
+        except Exception as e:
+            LOG.error(
+                f"Setup failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            )
+
+        if not contract_is_setup:
+            for gas_payer, gas_payer_priv in self.multi_credentials:
+                txn_info = {
+                    "gas_payer": gas_payer,
+                    "gas_payer_priv": gas_payer_priv,
+                    "gas": gas
+                }
+                try:
+                    txn_func = self.job_contract.functions.setup
+                    func_args = [
+                        reputation_oracle, recording_oracle,
+                        reputation_oracle_stake, recording_oracle_stake,
+                        self.manifest_url, self.manifest_hash
+                    ]
+                    handle_transaction(txn_func, *func_args, **txn_info)
+                    return self.status() == Status.Pending and self.balance(
+                    ) == hmt_amount
+                except Exception as e:
+                    LOG.error(
+                        f"Setup failed with: {gas_payer} and {gas_payer_priv} due to {e}. Raffling new ones..."
+                    )
         return self.status() == Status.Pending and self.balance() == hmt_amount
 
     def add_trusted_handlers(self,
@@ -542,9 +606,6 @@ class Job:
         True
         >>> job.setup()
         True
-        >>> trusted_handlers = ['0x61F9F0B31eacB420553da8BCC59DC617279731Ac', '0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809']
-        >>> job.add_trusted_handlers(trusted_handlers)
-        True
         >>> payouts = [("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", Decimal('20.0')), ("0x852023fbb19050B8291a335E5A83Ac9701E7B4E6", Decimal('50.0'))]
         >>> job.bulk_payout(payouts, {}, rep_oracle_pub_key)
         True
@@ -769,9 +830,20 @@ class Job:
         True
         >>> job.setup()
         True
-        >>> trusted_handlers = ['0x61F9F0B31eacB420553da8BCC59DC617279731Ac', '0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809']
-        >>> job.add_trusted_handlers(trusted_handlers)
+        >>> job.store_intermediate_results(results, rep_oracle_pub_key)
         True
+
+        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
+        >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
+        >>> job.launch(rep_oracle_pub_key)
+        True
+        >>> job.setup()
+        True
+        >>> results = {"results": False}
+
+        Inject wrong credentials on purpose to test out raffling
+        >>> job.gas_payer_priv = "657b6497a355a3982928d5515d48a84870f057c4d16923eb1d104c0afada9aa8"
+        >>> job.multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
         >>> job.store_intermediate_results(results, rep_oracle_pub_key)
         True
 
@@ -1290,24 +1362,27 @@ class Job:
             gas
         })
 
-    def _create_escrow(self, gas: int = GAS_LIMIT) -> bool:
+    def _create_escrow(self,
+                       trusted_handlers=[],
+                       gas: int = GAS_LIMIT) -> bool:
         """Launches a new escrow contract to the ethereum network.
 
+        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
+        >>> trusted_handlers = [addr for addr, priv_key in multi_credentials]
         >>> credentials = {
         ... 	"gas_payer": "0x1413862C2B7054CDbfdc181B83962CB0FC11fD92",
         ... 	"gas_payer_priv": "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"
         ... }
         >>> job = Job(credentials, manifest)
-        >>> job._create_escrow()
+        >>> job._create_escrow(trusted_handlers)
         True
 
-        >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
         >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
 
         Inject wrong credentials on purpose to test out raffling
         >>> job.gas_payer_priv = "657b6497a355a3982928d5515d48a84870f057c4d16923eb1d104c0afada9aa8"
         >>> job.multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "28e516f1e2f99e96a48a23cea1f94ee5f073403a1c68e818263f0eb898f1c8e5"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
-        >>> job._create_escrow()
+        >>> job._create_escrow(trusted_handlers)
         True
 
         Args:
@@ -1322,13 +1397,14 @@ class Job:
         """
         try:
             txn_func = self.factory_contract.functions.createEscrow
+            txn_args = [trusted_handlers]
             txn_info = {
                 "gas_payer": self.gas_payer,
                 "gas_payer_priv": self.gas_payer_priv,
                 "gas": gas
             }
 
-            handle_transaction(txn_func, *[], **txn_info)
+            handle_transaction(txn_func, *txn_args, **txn_info)
             return True
         except Exception as e:
             LOG.error(
@@ -1343,8 +1419,9 @@ class Job:
                 "gas_payer_priv": gas_payer_priv,
                 "gas": gas
             }
+            txn_args = [trusted_handlers]
             try:
-                handle_transaction(txn_func, *[], **txn_info)
+                handle_transaction(txn_func, *txn_args, **txn_info)
                 self.gas_payer = gas_payer
                 self.gas_payer_priv = gas_payer_priv
                 escrow_created = True
